;- PB v4 code added to make the older syntaxhighlighting run with CAV
  Macro MemoryAsciiLength(Memory)
    MemoryStringLength(Memory, #PB_Ascii)
  EndMacro
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

; the following are repeated declarations, for the case
; that the engine is used standalone
;
Global Dim ValidCharacters.b(255)  ; used by Syntax Highlighting and Structure Viewer

; OS specific hilightning color representation:
;
Global *ASMKeywordColor, *BackgroundColor, *BasicKeywordColor, *CommentColor, *ConstantColor, *LabelColor
Global *NormalTextColor, *NumberColor, *OperatorColor, *PointerColor, *PureKeywordColor, *SeparatorColor
Global *StringColor, *StructureColor, *LineNumberColor, *LineNumberBackColor, *MarkerColor, *CurrentLineColor, *CursorColor, *SelectionColor

Global *ActiveSource.SourceFile
Global EnableColoring
Global EnableCaseCorrection 
Global EnableKeywordBolding 


;#NbASMKeywords = 232
Global NbASMKeywords   ; will be later filled directly by the DataSection reading rountine...
#NbBasicKeywords = 90

Global Dim BasicKeywordsHT.l(255)
Global Dim BasicKeywords.s(#NbBasicKeywords)
Global Dim BasicKeywordsReal.s(#NbBasicKeywords)
Global Dim BasicKeywordsEndKeywords.s(#NbBasicKeywords)
Global Dim BasicKeywordsSpaces.s(#NbBasicKeywords)

;Global Dim ASMKeywords.s(#NbASMKeywords)
Global Dim ASMKeywordsHT.l(255)

;Dim BasicFunctionsHT.l(255)
;Dim APIFunctionsHT.l(255)

Global Dim QuickHelpStack$(200) ; to be able to display the QuickHelp to the function where the cursor is currently inside, even when it is not over it

Global BasicKeyword$, ASMKeyword$
Global NbBasicFunctions, NbApiFunctions

Global Dim TriggerCharacters.b(255) ; chars, that trigger an update of the current line when typed.

Structure HilightPTR
  StructureUnion
    b.b
    c.c
    w.w
    l.l
    s.s
    f.f
    nb.b[10]  ; access the next bytes
    nc.c[10]
  EndStructureUnion
EndStructure

Procedure ByteUcase(Byte.b) ; get the upper case value of a char in ascii mode
  If Byte >= 'a' And Byte <= 'z'
    ProcedureReturn Byte - 'a' + 'A'
  Else
    ProcedureReturn Byte & $FF
  EndIf
EndProcedure


Procedure InitSyntaxCheckArrays()
  Protected k
  ; Build the ValidCharacter table to have have a faster routine to detect
  ; if the char is a considered as a string or not. Nice improvement :)
  ;
  For k='0' To '9'            ; Tell that the ASCII from 0 to 9 are valid
    ValidCharacters(k) = 1
  Next

  For k='A' To 'Z'            ; Tell that the ASCII from A to Z are valid
    ValidCharacters(k) = 1
  Next

  For k='a' To 'z'            ; Tell that the ASCII from a to z are valid
    ValidCharacters(k) = 1
  Next

  ValidCharacters('_') = 1

  ; Set the characters that trigger a line update:
  ; Note: Space also triggers an update, but only on the first space, so it is handled differently
  ;
  TriggerCharacters('(') = 1
  TriggerCharacters(')') = 1
  TriggerCharacters('\') = 1
  TriggerCharacters(':') = 1
  TriggerCharacters('.') = 1
  TriggerCharacters(',') = 1
  TriggerCharacters('[') = 1
  TriggerCharacters(']') = 1
  TriggerCharacters('=') = 1
  TriggerCharacters('+') = 1
  TriggerCharacters('-') = 1
  TriggerCharacters('*') = 1
  TriggerCharacters('/') = 1
  TriggerCharacters('&') = 1
  TriggerCharacters('|') = 1
  TriggerCharacters('!') = 1
  TriggerCharacters('~') = 1
  TriggerCharacters('<') = 1
  TriggerCharacters('>') = 1

EndProcedure
  

Procedure InitSyntaxHilightning()
  Static *PBFunctionsBuffer, *ApiFunctionsBuffer
  Protected CurrentChar, k, Char, NbPBFunctions
  ; Now, init the internal PureBasic keywords
  ;
  CurrentChar = 0
  Restore BasicKeywords
  For k=1 To #NbBasicKeywords
    Read.s BasicKeywordsReal(k)
    Read.s BasicKeywords(k)
    Read.s BasicKeywordsEndKeywords(k)
    Read.s BasicKeywordsSpaces(k)
    
    Char = Asc(BasicKeywords(k))
    If Char <> CurrentChar
      BasicKeywordsHT(Char) = k
      CurrentChar = Char
    EndIf
  Next


  ; And next, init the ASM keywords
  ;
  CurrentChar = 0
  Restore ASMKeywords
  
  Read NbASMKeywords
  
  Global Dim ASMKeywords.s(NbASMKeywords)
  
  For k=1 To NbASMKeywords
    Read.s ASMKeywords(k)
    
    Char = Asc(ASMKeywords(k))
    If Char <> CurrentChar
      ASMKeywordsHT(Char) = k
      CurrentChar = Char
    EndIf
  Next  



; ***************************************************************************
; Following code is not needed, because the original PB functions are already
; included in CAV, see CodeArchivViewer_PBFunctions.pb file.

  ; We only need the 
  NbPBFunctions = #NbPBFunctions 
; ***************************************************************************

;   ; Read the PB Functions file
;   ;
;   If *PBFunctionsBuffer
;     FreeMemory(*PBFunctionsBuffer) ; in case of compiler restart
;     *PBFunctionsBuffer = 0
;   EndIf
; 
;   NbBasicFunctions = 0
;   Dim *BasicFunctionsArray(0, 1)
; 
;   If ReadFile(#FILE_LoadFunctions, TempPath$ + #DEFAULT_FunctionFile)
; 
;     NbBasicFunctions = Val(ReadString())
;     If NbBasicFunctions < 6000 And NbBasicFunctions > 0  ; Sanity check... If over, it's a bit strange...
;           
;       *BasicFunctionsBuffer = AllocateMemory(Lof()+2) 
;       If *BasicFunctionsBuffer
;       
;         *Cursor.Byte = *BasicFunctionsBuffer
;         ReadData(*Cursor, Lof())
;     
;         Dim *BasicFunctionsArray(NbBasicFunctions, 1)
;         CurrentFunction = 0
; 
;         Repeat
;                 
;           *BasicFunctionsArray(CurrentFunction, 0) = *Cursor  ; FunctionName
;         
;           While *Cursor\b <> ' ' And *Cursor\b <> 10
;             *Cursor+1
;           Wend
;           
;           If *Cursor\b = 10  ; End of line
;             *Cursor\b = 0
;             *Cursor+1
;           
;             *BasicFunctionsArray(CurrentFunction, 1) = @""  ; QuickHelp
;           Else
;             *Cursor\b = 0
;             *Cursor+1
;           
;             *BasicFunctionsArray(CurrentFunction, 1) = *Cursor  ; QuickHelp
;                         
;             While *Cursor\b <> 10 And *Cursor\b ; Skip the end of line
;               *Cursor+1
;             Wend
;             
;             *Cursor\b = 0
;             *Cursor+1
;           EndIf
;           
;           If (*Cursor\b = 10)
;             *Cursor+1
;           EndIf
;           
;           CurrentFunction+1
;         Until *Cursor\b = 0 Or CurrentFunction > NbBasicFunctions
;       
;       Else
;         NbBasicFunctions = 0
; 
;       EndIf
;     EndIf  
;    
;     CloseFile(#FILE_LoadFunctions)  
;   EndIf
; 
; 
;   ; PureBasic function Hash Table
;   ;
;   CurrentChar = 0
;   For k=1 To NbBasicFunctions
;     Char = Asc(UCase(PeekS(*BasicFunctionsArray(k-1,0))))
;     If Char <> CurrentChar
;       BasicFunctionsHT(Char) = k
;       CurrentChar = Char
;     EndIf
;   Next
; 



; ***************************************************************************
; Following code is not needed, because the original PB functions are already
; included in CAV, see CodeArchivViewer_WinAPIFunctions.pb file.

  ; We only need the 
  NbAPIFunctions = #NbWinAPIFunctions
; ***************************************************************************

;   ; Read the API Functions file
;   ;
;   If *APIFunctionsBuffer = 0  ; only needed once, not on compiler restart
; 
;     NbAPIFunctions = 0
  Global Dim *APIFunctionsArray(0, 1)
; 
;     If ReadFile(#FILE_LoadAPI, PureBasicPath$ + #DEFAULT_ApiFile)
; 
;       NbAPIFunctions = Val(ReadString())
; 
;       *Cursor.Byte = AllocateMemory(Lof()+2) ; this one never changes, because the API list isn't compiler generated
;       *APIFunctionsBuffer = *Cursor
;       If *Cursor
;         ReadData(*Cursor, Lof())
;   
;         Dim *APIFunctionsArray(NbAPIFunctions, 1)
;         CurrentFunction = 0
; 
;         Repeat
;               
;           *APIFunctionsArray(CurrentFunction, 0) = *Cursor  ; FunctionName
;       
;           While *Cursor\b <> ' ' And *Cursor\b <> 13
;             *Cursor+1
;           Wend
;           
;           If *Cursor\b = 13  ; End of line
;             *Cursor\b = 0
;             *Cursor+1
;         
;             *APIFunctionsArray(CurrentFunction, 1) = 0  ;  no QuickHelp
;           Else
;             *Cursor\b = 0
;             *Cursor+1
;         
;             *APIFunctionsArray(CurrentFunction, 1) = *Cursor  ; QuickHelp
;                       
;             While *Cursor\b <> 13 And *Cursor\b ; Skip the end of line
;               *Cursor+1
;             Wend
;            
;             *Cursor\b = 0
;             *Cursor+1
;           EndIf
;         
;           If (*Cursor\b = 10)
;             *Cursor+1
;           EndIf
;         
;           CurrentFunction+1
;         Until *Cursor\b = 0 Or CurrentFunction > NbAPIFunctions
; 
;       EndIf
;     
;       CloseFile(#FILE_LoadAPI)
;     EndIf
; 
;     ; API function Hash Table
;     ;
;     CurrentChar = 0
;     For k=1 To NbAPIFunctions
;       Char = Asc(UCase(PeekS(*APIFunctionsArray(k-1,0))))
;       If Char <> CurrentChar
;         APIFunctionsHT(Char) = k
;         CurrentChar = Char
;       EndIf
;     Next
; 
;   EndIf




  ; Indicate that hilightning is ready now: 
  ;
  ;IsHilightningReady = 1
  
EndProcedure

; do no longer work on strings but only on buffers, to avoid unneeded ascii-unicode conversions
;
Procedure IsAPIFunction(Word$)
  Protected Result, *word, k, quit, *winapifunction, Compare
  
  Result = -1
  
  *word = @Word$

  If Right(Word$, 1) = "_"
    Word$ = Left(Word$, Len(Word$)-1)
  
    k = APIFunctionsHT(Asc(UCase(Word$)))  ; We use an HashTable to get access to the index very quickly
    If k
      k-1
      While Quit = 0 And k < NbAPIFunctions
      
        ;Compare = CompareMemoryString(*APIFunctionsArray(k,0), Word$, 1)  ; Case insensitive compare
        ;Compare = CompareMemoryString(@WinAPIFunctions(k), Word$, 1)  ; Case insensitive compare
        *winapifunction = @WinAPIFunctions(k)
        Compare = CompareMemoryString(*winapifunction, *word, 1)  ; Case insensitive compare
      
        If Compare <= 0
          If Compare = 0
            Result = k
            Quit   = 1
          EndIf
        Else
          Quit = 1
        EndIf

        k+1
      Wend
    EndIf
  
  EndIf
    
  ProcedureReturn Result
EndProcedure

Procedure IsBasicFunction(Word$)
  Protected Result, *word, k, quit, compare, *pbfunction
  
  Result = -1
  
  *word = @Word$
  
  ;k = BasicFunctionsHT(Asc(UCase(Word$)))  ; We use an HashTable to get access to the index very quickly
  k = PBFunctionsHT(Asc(UCase(Word$)))  ; We use an HashTable to get access to the index very quickly

  If k
    k-1
    While Quit = 0 And k < NbBasicFunctions
    
;      Compare = CompareMemoryString(*BasicFunctionsArray(k,0), Word$, 1)  ; Case insensitive compare
      *pbfunction = @PBFunctions(k)
      Compare = CompareMemoryString(*pbfunction, *word, 1)  ; Case insensitive compare
      ; Compare = CompareMemoryString(@PBFunctions(k), Word$, 1)  ; Case insensitive compare
    
      If Compare <= 0
        If Compare = 0
          Result = k
          Quit   = 1
        EndIf
      Else
        Quit = 1
      EndIf

      k+1
    Wend
  EndIf
  
  ProcedureReturn Result
EndProcedure


Procedure IsASMKeyword(Word$)
  Protected k, quit, Result

  Word$ = UCase(Word$)
  k = ASMKeywordsHT(Asc(Word$))  ; We use an HashTable to get access to the index very quickly
  If k
    While Quit = 0 And k <= NbASMKeywords
      
      If ASMKeywords(k) <= Word$
        If ASMKeywords(k) = Word$
          ASMKeyword$ = ASMKeywords(k)
          Result = k
        EndIf
      Else
        Quit = 1
      EndIf

      k+1
    Wend
  EndIf

  ProcedureReturn Result
EndProcedure


Procedure IsBasicKeyword(Word$)
  Protected k, quit, Result
  Word$ = LCase(Word$)
  k = BasicKeywordsHT(Asc(Word$))  ; We use an HashTable to get access to the index very quickly
  If k
    While Quit = 0 And k <= #NbBasicKeywords
      
      If BasicKeywords(k) <= Word$
        If BasicKeywords(k) = Word$
          BasicKeyword$ = BasicKeywordsReal(k)
          Result = k
        EndIf
      Else
        Quit = 1
      EndIf

      k+1
    Wend
  EndIf

  ProcedureReturn Result
EndProcedure




Procedure IsDecNumber(SearchString$)
  Protected *string.BYTE
  *string.BYTE = @SearchString$
  While (*string\b & $FF) <> 0
    If (*string\b & $FF) < 48 Or (*string\b & $FF) > 57
      ProcedureReturn 0
    EndIf
    *string + 1
  Wend
  ProcedureReturn 1
EndProcedure




; callback function
;
;Declare HilightCallback(*StringStart.BYTE, Length, *Color, IsBold, TextChanged)

Procedure HilightningEngine(*InBuffer, InBufferLength, CursorPosition, *HilightCallback, IsSourceCode)
  Protected *Cursor.HilightPTR, *InBufferEnd, SeperatorChar, LineStart, QuickHelpPosition, *StringStart
  Protected *WordStart, *WordEnd.Byte, AfterSpaces, WordStart$, SeparatorUsed, IgnoreSeparator, SeparatorChar
  Protected NewLine, TextChanged, FunctionPosition, OldSeparatorChar, *ForwardCursor.HilightPTR, NextWord$
  Protected *BackCursor.HilightPTR, IsPointer, *CheckEnd
  
  *Cursor.HilightPTR = *InBuffer
  *InBufferEnd = *InBuffer + InBufferLength

  SeperatorChar = 0
  LineStart = 1

  QuickHelpPosition = 0
  QuickHelpStack$(0) = ""

  While *Cursor < *InBufferEnd

    *StringStart = *Cursor

    ; first, skip all tabs and spaces
    While *Cursor < *InBufferEnd And (*Cursor\b = ' ' Or *Cursor\b = 9)
      *Cursor + 1
    Wend

    ; Now, isolate the word..

    *WordStart = *Cursor

    While *Cursor < *InBufferEnd And ValidCharacters(*Cursor\b & $FF)
      *Cursor + 1
    Wend

    *WordEnd.Byte = *Cursor


    ; Skip all the spaces after the word..

    AfterSpaces = 0;
    While *Cursor < *InBufferEnd And (*Cursor\b = ' ' Or *Cursor\b = 9)
      AfterSpaces+1
      *Cursor + 1
    Wend    

    WordStart$ = PeekS(*WordStart, *WordEnd-*WordStart)
    SeparatorUsed = 0
    IgnoreSeparator = 0

    ; Here we got our separator: (, ', ;....). If it's a valid character, then the separator was spaces..
    
    If *Cursor > *InBufferEnd 
      SeparatorChar = 3 ; indicate end of the loop!
      
    Else

      NewLine = 0
      If ValidCharacters(*Cursor\b & $FF)
        SeparatorChar = 0

      ElseIf *Cursor\b = 13 Or *Cursor\b = 10 
        NewLine = 1

        If *Cursor+1 < *InBufferEnd And (*Cursor\nb[1] = 13 Or *Cursor\nb[1] = 10)  ; output the newline directly with the word
          *Cursor + 2
        Else
          *Cursor + 1
        EndIf

        SeparatorChar = 1
        
      ElseIf *Cursor\b = 0
        SeparatorChar = 3 ; indicate end of the loop!
        
      Else
        SeparatorChar = *Cursor\b & $FF
      EndIf
    EndIf
    
    
    
    If IsBasicKeyword(WordStart$)

      ; -------------------- Basic Keywords -------------------------

      If EnableCaseCorrection
        CopyMemory(@BasicKeyword$, *WordStart, Len(BasicKeyword$)) ; no PokeS() as it will write a 0!
        CallFunctionFast(*HilightCallback, *StringStart, *Cursor-*StringStart, *BasicKeywordColor, EnableKeywordBolding, 1)        
      Else
        CallFunctionFast(*HilightCallback, *StringStart, *Cursor-*StringStart, *BasicKeywordColor, EnableKeywordBolding, 0)
      EndIf



    ElseIf IsSourceCode And *ActiveSource\EnableASM And IsASMKeyword(WordStart$)
     
      ; -------------------- ASM Keywords ---------------------------

      If EnableCaseCorrection
        CopyMemory(@ASMKeyword$, *WordStart, Len(ASMKeyword$)) ; no PokeS() as it will write a 0!
        CallFunctionFast(*HilightCallback, *StringStart, *Cursor-*StringStart, *ASMKeywordColor, 0, 1)         
      Else
        CallFunctionFast(*HilightCallback, *StringStart, *Cursor-*StringStart, *ASMKeywordColor, 0, 0)       
      EndIf




    ElseIf SeparatorChar = ':' And LineStart And WordStart$ <> ""

      ; --------------------- Labels -------------------------------
      
      CallFunctionFast(*HilightCallback, *StringStart, *Cursor-*StringStart+1, *LabelColor, 0, 0) ; include the : !
      SeparatorUsed = 1
      


    ElseIf SeparatorChar = '(' And WordStart$ <> ""

      ; --------------------- Procedures / Functions --------------------
      TextChanged = 0

      FunctionPosition = IsAPIFunction(WordStart$)
      If FunctionPosition > -1

        If *Cursor - *InBuffer < CursorPosition      
          ;QuickHelpStack$(QuickHelpPosition) = PeekS(*APIFunctionsArray(FunctionPosition, 0))
          QuickHelpStack$(QuickHelpPosition) = PeekS(@WinAPIFunctions(FunctionPosition))
        EndIf
        
        If EnableCaseCorrection
          ;CopyMemory(*APIFunctionsArray(FunctionPosition, 0), *WordStart, MemoryStringLength(*APIFunctionsArray(FunctionPosition, 0)))
          CopyMemory(@WinAPIFunctions(FunctionPosition), *WordStart, MemoryStringLength(@WinAPIFunctions(FunctionPosition)))
          TextChanged = 1
        EndIf

      Else
        FunctionPosition = IsBasicFunction(WordStart$)
        If FunctionPosition > -1

          If *Cursor - *InBuffer < CursorPosition
            ;QuickHelpStack$(QuickHelpPosition) = PeekS(*BasicFunctionsArray(FunctionPosition, 0))
            QuickHelpStack$(QuickHelpPosition) = PeekS(@PBFunctions(FunctionPosition))
          EndIf

          If EnableCaseCorrection
            ;CopyMemory(*BasicFunctionsArray(FunctionPosition, 0), *WordStart, MemoryStringLength(*BasicFunctionsArray(FunctionPosition, 0)))
            CopyMemory(@PBFunctions(FunctionPosition), *WordStart, MemoryStringLength(@PBFunctions(FunctionPosition)))
            TextChanged = 1
          EndIf
        EndIf         

      EndIf

      If OldSeparatorChar = '.'
        WordStart$ = UCase(WordStart$)
        If WordStart$="L" Or WordStart$="W" Or WordStart$="B" Or WordStart$="F" Or WordStart$="S" 
          CallFunctionFast(*HilightCallback, *StringStart, *Cursor-*StringStart, *NormalTextColor, 0, TextChanged)
        Else
          CallFunctionFast(*HilightCallback, *StringStart, *Cursor-*StringStart, *StructureColor, 0, TextChanged)
        EndIf     
      Else
        CallFunctionFast(*HilightCallback, *StringStart, *Cursor-*StringStart, *PureKeywordColor, 0, TextChanged)
      EndIf     
  

    ElseIf IsDecNumber(WordStart$)

      ; ---------------------- Decimal Numbers -----------------------

      If SeparatorChar = '.'
        CallFunctionFast(*HilightCallback, *StringStart, *Cursor-*StringStart+1, *NumberColor, 0, 0) ; include the .
        SeparatorUsed = 1
      Else
        CallFunctionFast(*HilightCallback, *StringStart, *Cursor-*StringStart, *NumberColor, 0, 0)
      EndIf


    ElseIf SeparatorChar = '$' And AfterSpaces = 0
 
      ; ---------------------- String$ Variables ----------------------

      CallFunctionFast(*HilightCallback, *StringStart, *Cursor-*StringStart+1, *NormalTextColor, 0, 0) ; include the $
      SeparatorUsed = 1
      


    ElseIf OldSeparatorChar = '\' Or SeparatorChar = '\' 

      ; ---------------------- Structures --------------------------
      CallFunctionFast(*HilightCallback, *StringStart, *Cursor-*StringStart, *StructureColor, 0, 0)



    ElseIf SeparatorChar = '.' Or OldSeparatorChar = '.'

      ; ---------------------- Structures Or PB Types -----------------

      If OldSeparatorChar = '.'
        WordStart$ = UCase(WordStart$)
        If WordStart$="L" Or WordStart$="W" Or WordStart$="B" Or WordStart$="F" Or WordStart$="S"
          CallFunctionFast(*HilightCallback, *StringStart, *Cursor-*StringStart, *NormalTextColor, 0, 0) 
        Else
          CallFunctionFast(*HilightCallback, *StringStart, *Cursor-*StringStart, *StructureColor, 0, 0)
        EndIf

      Else
        *ForwardCursor.HilightPTR = *Cursor+1
        While *ForwardCursor < *InBufferEnd And ValidCharacters(*ForwardCursor\b & $FF)
          *ForwardCursor + 1
        Wend
        NextWord$ = UCase(PeekS(*Cursor+1, *ForwardCursor-*Cursor-1))

        If *ForwardCursor\b = '('  ; Dim whatever.xy() or NewList...
          CallFunctionFast(*HilightCallback, *StringStart, *Cursor-*StringStart, *PureKeywordColor, 0, 0)
       
        ElseIf NextWord$ = "B" Or NextWord$ = "W" Or NextWord$ = "L" Or NextWord$ = "S" Or NextWord$ = "F"
          CallFunctionFast(*HilightCallback, *StringStart, *Cursor-*StringStart, *NormalTextColor, 0, 0)

        Else
          CallFunctionFast(*HilightCallback, *StringStart, *Cursor-*StringStart, *StructureColor, 0, 0)

        EndIf
      EndIf


    ElseIf *Cursor-*StringStart > 0 

      ; ---------------------- Normal Text -------------------------

      CallFunctionFast(*HilightCallback, *StringStart, *Cursor-*StringStart, *NormalTextColor, 0, 0)

    EndIf

    ; Everything that has a Word before a separator has been processed and outputed now.
    ; Only separators remain

    If SeparatorUsed = 0 And SeparatorChar > 1

      If SeparatorChar = '!' And LineStart

        ; --------------------- ASM Line with "!" ---------------------

        *StringStart = *Cursor
        While *Cursor < *InBufferEnd And (*Cursor\b <> 10 And *Cursor\b <> 13 And *Cursor\b)
          *Cursor + 1
        Wend
        NewLine = 1

        If *Cursor+1 < *InBufferEnd And (*Cursor\nb[1] = 10 Or *Cursor\nb[1] = 13)
          *Cursor + 1
        EndIf
        
        If *Cursor < *InBufferEnd ; include the newline
          *Cursor + 1        
        EndIf        

        CallFunctionFast(*HilightCallback, *StringStart, *Cursor-*StringStart, *ASMKeywordColor, 0, 0) 
        SeparatorChar = 2



      ElseIf SeparatorChar = '"'
    
        ; --------------------- Strings ----------------------------

        *StringStart = *Cursor ; the position of the Separator
        *Cursor + 1
        While *Cursor < *InBufferEnd And (*Cursor\b <> '"' And *Cursor\b <> 10 And *Cursor\b <> 13 And *Cursor\b)
          *Cursor + 1
        Wend

        If *Cursor\b = 10 Or *Cursor\b = 13
          NewLine = 1
          
          If *Cursor+1 < *InBufferEnd And (*Cursor\nb[1] = 10 Or *Cursor\nb[1] = 13)
            *Cursor + 1
          EndIf
        EndIf
        
        If *Cursor < *InBufferEnd ; include the newline
          *Cursor + 1        
        EndIf        

        CallFunctionFast(*HilightCallback, *StringStart, *Cursor-*StringStart, *StringColor, 0, 0) ; include the second " !
        SeparatorChar = 2

      ElseIf SeparatorChar = ';'



        ; ------------------- Comments -------------------------

        *StringStart = *Cursor
        While *Cursor < *InBufferEnd And (*Cursor\b <> 10 And *Cursor\b <> 13 And *Cursor\b)
          *Cursor + 1
        Wend
        NewLine = 1

        If *Cursor+1 < *InBufferEnd And (*Cursor\nb[1] = 10 Or *Cursor\nb[1] = 13)
          *Cursor + 1
        EndIf
        
        If *Cursor < *InBufferEnd ; include the newline
          *Cursor + 1        
        EndIf

        CallFunctionFast(*HilightCallback, *StringStart, *Cursor-*StringStart, *CommentColor, 0, 0) 
        SeparatorChar = 2



      ElseIf SeparatorChar = '#'

        ; ---------------------- Constants --------------------

        *StringStart = *Cursor
        *Cursor + 1
        While *Cursor < *InBufferEnd And ValidCharacters(*Cursor\b & $FF)
          *Cursor + 1
        Wend

        CallFunctionFast(*HilightCallback, *StringStart, *Cursor-*StringStart, *ConstantColor, 0, 0) ; don't include the current char!
        SeparatorChar = 2 ; don't move the cursor at the loop end



      ElseIf SeparatorChar = 39 ; '
      
        ; --------------------- Numeric Constants -----------------------
        
        *StringStart = *Cursor ; the position of the Separator
        *Cursor + 1
        While *Cursor < *InBufferEnd And (*Cursor\b <> 39 And *Cursor\b <> 10 And *Cursor\b <> 13 And *Cursor\b)
          *Cursor + 1
        Wend

        If *Cursor < *InBufferEnd And (*Cursor\b = 10 Or *Cursor\b = 13)
          NewLine = 1
          
          If *Cursor+1 < *InBufferEnd And (*Cursor\nb[1] = 10 Or *Cursor\nb[1] = 13)
            *Cursor + 1
          EndIf
        EndIf

        CallFunctionFast(*HilightCallback, *StringStart, *Cursor-*StringStart+1, *ConstantColor, 0, 0) ; include the second ' !
        *Cursor + 1
        SeparatorChar = 2        



      ElseIf SeparatorChar = '$'

        ; -------------------------- HEX Values ----------------------------

        *StringStart = *Cursor
        *Cursor + 1
        While *Cursor < *InBufferEnd And (ValidCharacters(*Cursor\b & $FF) And *Cursor\b & $FF < 'G')
          *Cursor + 1
        Wend

        CallFunctionFast(*HilightCallback, *StringStart, *Cursor-*StringStart, *NumberColor, 0, 0) ; don't include the current char
        SeparatorChar = 2

      
      ElseIf SeparatorChar = '%'

        ; ---------------------------- % - binary or mod ------------------------

        *StringStart = *Cursor
        *Cursor + 1

        While *Cursor < *InBufferEnd And (*Cursor\b = '1' Or *Cursor\b = '0')
          *Cursor + 1
        Wend

        If *Cursor = *StringStart + 1   ; no numbers found, so not binary
          CallFunctionFast(*HilightCallback, *StringStart, 1, *OperatorColor, 0, 0)

        Else  ; binary
          CallFunctionFast(*HilightCallback, *StringStart, *Cursor-*StringStart, *NumberColor, 0, 0)

        EndIf  
        SeparatorChar = 2


      ElseIf SeparatorChar = '@' Or SeparatorChar = '?'

        ; ------------------------------ Pointer @ or ? --------------------

        *StringStart = *Cursor
        *Cursor + 1
        While *Cursor < *InBufferEnd And (ValidCharacters(*Cursor\b & $FF) Or *Cursor\b = '$')
          *Cursor + 1
        Wend

        CallFunctionFast(*HilightCallback, *StringStart, *Cursor-*StringStart, *PointerColor, 0, 0) ; don't include the current char!
        SeparatorChar = 2 ; don't move the cursor at the loop end



      ElseIf SeparatorChar = '*'

        ; ---------------------- * - Pointer or Operator -----------------------------

        If ValidCharacters(*Cursor\nb[1] & $FF) = 0 ; surely an operator
          CallFunctionFast(*HilightCallback, *Cursor, 1, *OperatorColor, 0, 0)
           
        Else ; since 5*a is not a pointer, we need to further check..

          *BackCursor.HilightPTR = *Cursor-1
          IsPointer = 1
          While *BackCursor >= *InBuffer And *BackCursor\b <> 10 And *BackCursor\b <> 13
            If *BackCursor\b = '(' Or *BackCursor\b = ':' Or *BackCursor\b = '[' Or *BackCursor\b = ',' Or *BackCursor\b = '*' Or *BackCursor\b = '=' Or *BackCursor\b = '+' Or *BackCursor\b = '-' Or *BackCursor\b = '/' Or *BackCursor\b = '@' Or *BackCursor\b = '&' Or *BackCursor\b = '|' Or *BackCursor\b = '!' Or *BackCursor\b = '~' Or *BackCursor\b = '<' Or *BackCursor\b = '>' Or *BackCursor\b = '\'
              ; must be a pointer
              Break     
            ElseIf ValidCharacters(*BackCursor\b & $FF) Or *BackCursor\b = ')' Or *BackCursor\b = ']' Or *BackCursor\b = '.'
              ; can't be a pointer
              IsPointer = 0
              Break
            EndIf
            *BackCursor - 1
          Wend

          If ValidCharacters(*BackCursor\b & $FF)  ; in this case, check if this was a PB Keyword
            *CheckEnd = *BackCursor
            While *BackCursor >= *InBuffer And ValidCharacters(*BackCursor\b & $FF) 
              *BackCursor - 1
            Wend
            IsPointer = IsBasicKeyword(PeekS(*BackCursor+1, *CheckEnd-*BackCursor))
          EndIf

          If IsPointer = 0
            CallFunctionFast(*HilightCallback, *Cursor, 1, *OperatorColor, 0, 0)

          Else
            *StringStart = *Cursor
            *Cursor + 1
            While *Cursor < *InBufferEnd And (ValidCharacters(*Cursor\b & $FF) Or *Cursor\b = '$')
              *Cursor + 1
            Wend

            CallFunctionFast(*HilightCallback, *StringStart, *Cursor-*StringStart, *PointerColor, 0, 0)
            SeparatorChar = 2 ; don't move the cursor at the loop end

          EndIf

        EndIf


     

      ElseIf SeparatorChar = '=' Or SeparatorChar = '+' Or SeparatorChar = '-' Or SeparatorChar = '/' Or SeparatorChar = '&' Or SeparatorChar = '|' Or SeparatorChar = '!' Or SeparatorChar = '~' Or SeparatorChar = '<' Or SeparatorChar = '>'

        ; ------------------------- Operators --------------------------------
        ; = + - / & | ! ~ < > 

        CallFunctionFast(*HilightCallback, *Cursor, 1, *OperatorColor, 0, 0) 



      ElseIf SeparatorChar = '(' Or SeparatorChar = ')' Or SeparatorChar = '[' Or SeparatorChar = ']' Or SeparatorChar = '.' Or SeparatorChar = ',' Or SeparatorChar = ':' Or SeparatorChar = '\'

        ; ------------------------- Separators -------------------------------
        ; ( ) [ ] . , : \

        If *Cursor - *InBuffer < CursorPosition
          If SeparatorChar = '('     ; for the QuickHelp.. increase the stack on every ( and decreas it on )
            If QuickHelpPosition < 200
              QuickHelpPosition + 1
              QuickHelpStack$(QuickHelpPosition) = ""
            EndIf
          ElseIf SeparatorChar = ')'
            If QuickHelpPosition > 0
              QuickHelpPosition - 1   
              QuickHelpStack$(QuickHelpPosition) = ""
            EndIf
          EndIf
        EndIf

        CallFunctionFast(*HilightCallback, *Cursor, 1, *SeparatorColor, 0, 0) 
       
 
      ElseIf SeparatorChar > 3

        ; ----------------------- Normal Text Separator --------------

        CallFunctionFast(*HilightCallback, *Cursor, 1, *NormalTextColor, 0, 0) 
      EndIf
    EndIf

    If SeparatorChar > 2
      *Cursor + 1 ; move away from the separator char
    EndIf

    OldSeparatorChar = SeparatorChar
    
    If NewLine
      LineStart = 1

      If *Cursor - *InBuffer < CursorPosition   ; reset quickhelpstack on new line
        QuickHelpPosition = 0
        QuickHelpStack$(0) = ""
      EndIf
    Else
      LineStart = 0
    EndIf
    
    If SeparatorChar = 3 ; null byte encountered, quit the loop!
      Break
    EndIf
  Wend
  
; ******************* NOT NEEDED IN CAV *********************
;   If IsSourceCode
;
;     ; update the quickhelp
;     ;
;     While QuickHelpPosition > 0 And QuickHelpStack$(QuickHelpPosition) = ""  ; go back all empty spaces 
;       QuickHelpPosition - 1                                                     ; Example: Function( ( ( ...
;     Wend
;     
;     If QuickHelpPosition >= 0 And QuickHelpStack$(QuickHelpPosition) <> ""
;       Message$ = GenerateQuickHelpText(QuickHelpStack$(QuickHelpPosition))
;       ChangeStatus(Message$)
;     EndIf
;  
;   EndIf
; ************************************************************

EndProcedure


DataSection

  ;- Keywords - BASIC
      
  ; Note: First is the Keyword in real Case, second is the same in lowercase, then
  ; the corresponding end keyword (for autocomplete), then wether the tag should include a 
  ; trailing space in autocomplete (if enabled)
  ; Keywords must be sorted here!
  
  BasicKeywords:
    Data$ "And", "and", "", " "
    Data$ "As", "as", " ", " "

    Data$ "Break", "break", "", ""

    Data$ "CallDebugger"     , "calldebugger", "", ""
    Data$ "Case"             , "case", "", " "
    Data$ "CompilerCase"     , "compilercase", "", " "
    Data$ "CompilerDefault"  , "compilerdefault", "", ""
    Data$ "CompilerElse"     , "compilerelse", "", " "
    Data$ "CompilerEndIf"    , "compilerendif", "", ""
    Data$ "CompilerEndSelect", "compilerendselect", "", ""
    Data$ "CompilerError"    , "compilererror", " ", " "
    Data$ "CompilerIf"       , "compilerif", "CompilerEndIf", " "
    Data$ "CompilerSelect"   , "compilerselect", "CompilerEndSelect", " "
    Data$ "Continue"         , "continue", "", ""

    Data$ "Data"           , "data", "", " "
    Data$ "DataSection"    , "datasection", "EndDataSection", ""
    Data$ "Debug"          , "debug", "", " "
    Data$ "DebugLevel"     , "debuglevel", "", " "
    Data$ "Declare"        , "declare", "", ""
    Data$ "DeclareCDLL"    , "declarecdll", "", ""
    Data$ "DeclareDLL"     , "declaredll", "", ""
    Data$ "Default"        , "default", "", ""
    Data$ "Define"         , "define", " ", " "
    Data$ "Dim"            , "dim", "", " "
    Data$ "DisableDebugger", "disabledebugger", "", ""
    Data$ "DisableExplicit", "disableexplicit", "", " "

    Data$ "Else"              , "else", "", ""
    Data$ "ElseIf"            , "elseif", "", " "
    Data$ "EnableDebugger"    , "enabledebugger", "", ""
    Data$ "EnableExplicit"    , "enableexplicit", "", " "
    Data$ "End"               , "end", "", ""
    Data$ "EndDataSection"    , "enddatasection", "", ""
    Data$ "EndEnumeration"    , "endenumeration", "", ""
    Data$ "EndIf"             , "endif", "", ""
    Data$ "EndImport"         , "endimport", "", " "
    Data$ "EndInterface"      , "endinterface", "", ""
    Data$ "EndMacro"          , "endmacro", "", " "
    Data$ "EndProcedure"      , "endprocedure", "", ""
    Data$ "EndSelect"         , "endselect", "", ""
    Data$ "EndStructure"      , "endstructure", "", ""
    Data$ "EndStructureUnion" , "endstructureunion", "", ""
    Data$ "EndWith"           , "endwith", "", " "
    Data$ "Enumeration"       , "enumeration", "EndEnumeration", " "
    Data$ "Extends"           , "extends", "", " "

    Data$ "FakeReturn"   , "fakereturn", "", ""
    Data$ "For"          , "for", "Next", " "
    Data$ "ForEach"      , "foreach", "Next", " "
    Data$ "ForEver"      , "forever", "", ""

    Data$ "Global", "global", "", " "
    Data$ "Gosub" , "gosub", "", " "
    Data$ "Goto"  , "goto", "", " "

    Data$ "If"            , "if", "EndIf", " "
    Data$ "Import"        , "import", "EndImport", " "
    Data$ "ImportC"       , "importc", "EndImport", " "
    Data$ "IncludeBinary" , "includebinary", "", " "
    Data$ "IncludeFile"   , "includefile", "", " "
    Data$ "IncludePath"   , "includepath", "", " "
    Data$ "Interface"     , "interface", "EndInterface", " "

    Data$ "Macro", "macro", "EndMacro", " "

    Data$ "NewList", "newlist", "", " "
    Data$ "Next", "next", "", ""
    Data$ "Not"    , "not", " ", " "

    Data$ "Or", "or", "", " "

    Data$ "Procedure"      , "procedure", "EndProcedure", " "
    Data$ "ProcedureC"     , "procedurec", "EndProcedure", " "
    Data$ "ProcedureCDLL"  , "procedurecdll", "EndProcedure", " "
    Data$ "ProcedureDLL"   , "proceduredll", "EndProcedure", " "
    Data$ "ProcedureReturn", "procedurereturn", "", " "
    Data$ "Protected"      , "protected", "", " "
    Data$ "Prototype"      , "prototype", " ", " "
    Data$ "PrototypeC"     , "prototypec", " ", " "

    Data$ "Read", "read", "", " "
    Data$ "ReDim"  , "redim", " ", " "
    Data$ "Repeat", "repeat", "Until ", " "
    Data$ "Restore", "restore", "", " "
    Data$ "Return", "return", "", ""

    Data$ "Select", "select", "EndSelect", " "
    Data$ "Shared", "shared", "", " "
    Data$ "Static", "static", "", " "
    Data$ "Step", "step", "", " "
    Data$ "Structure", "structure", "EndStructure", " "
    Data$ "StructureUnion", "structureunion", "EndStructureUnion", ""
    Data$ "Swap"          , "swap", " ", " "
    
    Data$ "To", "to", "", " "

    Data$ "Until", "until", "", " "

    Data$ "Wend" , "wend", "", ""
    Data$ "While", "while", "Wend", " "
    Data$ "With" , "with", "EndWith", " "

    Data$ "XIncludeFile" , "xincludefile", "", " "
    Data$ "XOr"         , "xor", " ", " "



  ;- Keywords - ASM
  ASMKeywords:
  Data.l 393
  Data$ "AAA"
  Data$ "AAD"
  Data$ "AAM"
  Data$ "AAS"
  Data$ "ADC"
  Data$ "ADD"
  Data$ "AND"
  Data$ "ARPL"
  Data$ "BOUND"
  Data$ "BSF"
  Data$ "BSR"
  Data$ "BSWAP"
  Data$ "BT"
  Data$ "BTC"
  Data$ "BTR"
  Data$ "BTS"
  Data$ "CALL"
  Data$ "CBW"
  Data$ "CDQ"
  Data$ "CLC"
  Data$ "CLD"
  Data$ "CLI"
  Data$ "CLTS"
  Data$ "CMC"
  Data$ "CMOVA"
  Data$ "CMOVAE"
  Data$ "CMOVB"
  Data$ "CMOVBE"
  Data$ "CMOVC"
  Data$ "CMOVE"
  Data$ "CMOVG"
  Data$ "CMOVGE"
  Data$ "CMOVL"
  Data$ "CMOVLE"
  Data$ "CMOVNA"
  Data$ "CMOVNAE"
  Data$ "CMOVNB"
  Data$ "CMOVNBE"
  Data$ "CMOVNC"
  Data$ "CMOVNE"
  Data$ "CMOVNG"
  Data$ "CMOVNGE"
  Data$ "CMOVNL"
  Data$ "CMOVNLE"
  Data$ "CMOVNO"
  Data$ "CMOVNP"
  Data$ "CMOVNS"
  Data$ "CMOVNZ"
  Data$ "CMOVO"
  Data$ "CMOVP"
  Data$ "CMOVPE"
  Data$ "CMOVPO"
  Data$ "CMOVS"
  Data$ "CMOVZ"
  Data$ "CMP"
  Data$ "CMPS"
  Data$ "CMPSB"
  Data$ "CMPSD"
  Data$ "CMPSW"
  Data$ "CMPXCHG"
  Data$ "CMPXCHG8B"
  Data$ "CWD"
  Data$ "CWDE"
  Data$ "DAA"
  Data$ "DAS"
  Data$ "DB"
  Data$ "DD"
  Data$ "DEC"
  Data$ "DIV"
  Data$ "DW"
  Data$ "EMMS"
  Data$ "ENTER"
  Data$ "ESC"
  Data$ "F2XM1"
  Data$ "FABS"
  Data$ "FADD"
  Data$ "FADDP"
  Data$ "FBLD"
  Data$ "FBSTP"
  Data$ "FCHS"
  Data$ "FCLEX"
  Data$ "FCMOVB"
  Data$ "FCMOVBE"
  Data$ "FCMOVE"
  Data$ "FCMOVNB"
  Data$ "FCMOVNBE"
  Data$ "FCMOVNE"
  Data$ "FCMOVNU"
  Data$ "FCMOVU"
  Data$ "FCOM"
  Data$ "FCOMI"
  Data$ "FCOMIP"
  Data$ "FCOMP"
  Data$ "FCOMPP"
  Data$ "FCOS"
  Data$ "FDECSTP"
  Data$ "FDIV"
  Data$ "FDIVP"
  Data$ "FDIVR"
  Data$ "FDIVRP"
  Data$ "FFREE"
  Data$ "FIADD"
  Data$ "FICOM"
  Data$ "FICOMP"
  Data$ "FIDIV"
  Data$ "FIDIVR"
  Data$ "FILD"
  Data$ "FIMUL"
  Data$ "FINCSTP"
  Data$ "FINIT"
  Data$ "FIST"
  Data$ "FISTP"
  Data$ "FISUB"
  Data$ "FISUBR"
  Data$ "FLD"
  Data$ "FLD1"
  Data$ "FLDCW"
  Data$ "FLDENV"
  Data$ "FLDL2E"
  Data$ "FLDL2T"
  Data$ "FLDLG2"
  Data$ "FLDLN2"
  Data$ "FLDPI"
  Data$ "FLDZ"
  Data$ "FMUL"
  Data$ "FMULP"
  Data$ "FNCLEX"
  Data$ "FNINIT"
  Data$ "FNOP"
  Data$ "FNSAVE"
  Data$ "FNSTCW"
  Data$ "FNSTENV"
  Data$ "FNSTSW"
  Data$ "FPATAN"
  Data$ "FPREM"
  Data$ "FPREM1"
  Data$ "FPTAN"
  Data$ "FRNDINT"
  Data$ "FRSTOR"
  Data$ "FSAVE"
  Data$ "FSCALE"
  Data$ "FSETPM"
  Data$ "FSIN"
  Data$ "FSINCOS"
  Data$ "FSQRT"
  Data$ "FST"
  Data$ "FSTCW"
  Data$ "FSTENV"
  Data$ "FSTP"
  Data$ "FSTSW"
  Data$ "FSUB"
  Data$ "FSUBP"
  Data$ "FSUBR"
  Data$ "FSUBRP"
  Data$ "FTST"
  Data$ "FUCOM"
  Data$ "FUCOMI"
  Data$ "FUCOMIP"
  Data$ "FUCOMP"
  Data$ "FUCOMPP"
  Data$ "FWAIT"
  Data$ "FXAM"
  Data$ "FXCH"
  Data$ "FXTRACT"
  Data$ "FYL2X"
  Data$ "FYL2XP1"
  Data$ "HLT"
  Data$ "IDIV"
  Data$ "IMUL"
  Data$ "IN"
  Data$ "INC"
  Data$ "INS"
  Data$ "INSB"
  Data$ "INSD"
  Data$ "INSW"
  Data$ "INT"
  Data$ "INTO"
  Data$ "INVD"
  Data$ "INVLPG"
  Data$ "IRET"
  Data$ "IRETD"
  Data$ "JA"
  Data$ "JAE"
  Data$ "JB"
  Data$ "JBE"
  Data$ "JC"
  Data$ "JCXZ"
  Data$ "JE"
  Data$ "JECXZ"
  Data$ "JG"
  Data$ "JGE"
  Data$ "JL"
  Data$ "JLE"
  Data$ "JMP"
  Data$ "JNA"
  Data$ "JNAE"
  Data$ "JNB"
  Data$ "JNBE"
  Data$ "JNC"
  Data$ "JNE"
  Data$ "JNG"
  Data$ "JNGE"
  Data$ "JNL"
  Data$ "JNLE"
  Data$ "JNO"
  Data$ "JNP"
  Data$ "JNS"
  Data$ "JNZ"
  Data$ "JO"
  Data$ "JP"
  Data$ "JPE"
  Data$ "JPO"
  Data$ "JS"
  Data$ "JZ"
  Data$ "LAHF"
  Data$ "LAR"
  Data$ "LDS"
  Data$ "LEA"
  Data$ "LEAVE"
  Data$ "LES"
  Data$ "LFS"
  Data$ "LGDT"
  Data$ "LGS"
  Data$ "LIDT"
  Data$ "LLDT"
  Data$ "LMSW"
  Data$ "LOCK"
  Data$ "LODS"
  Data$ "LODSB"
  Data$ "LODSD"
  Data$ "LODSW"
  Data$ "LOOP"
  Data$ "LOOPE"
  Data$ "LOOPNE"
  Data$ "LOOPNZ"
  Data$ "LOOPZ"
  Data$ "LSL"
  Data$ "LSS"
  Data$ "LTR"
  Data$ "MOV"
  Data$ "MOVD"
  Data$ "MOVQ"
  Data$ "MOVS"
  Data$ "MOVSB"
  Data$ "MOVSD"
  Data$ "MOVSW"
  Data$ "MOVSX"
  Data$ "MOVZX"
  Data$ "MUL"
  Data$ "NEG"
  Data$ "NOP"
  Data$ "NOT"
  Data$ "OR"
  Data$ "OUT"
  Data$ "OUTS"
  Data$ "OUTSB"
  Data$ "OUTSD"
  Data$ "OUTSW"
  Data$ "PACKSSDW"
  Data$ "PACKSSWB"
  Data$ "PACKUSWB"
  Data$ "PADDB"
  Data$ "PADDD"
  Data$ "PADDSB"
  Data$ "PADDSW"
  Data$ "PADDUSB"
  Data$ "PADDUSW"
  Data$ "PADDW"
  Data$ "PAND"
  Data$ "PANDN"
  Data$ "PCMPEQB"
  Data$ "PCMPEQD"
  Data$ "PCMPEQW"
  Data$ "PCMPGTB"
  Data$ "PCMPGTD"
  Data$ "PCMPGTW"
  Data$ "PMADDWD"
  Data$ "PMULHW"
  Data$ "POP"
  Data$ "POPA"
  Data$ "POPAD"
  Data$ "POPF"
  Data$ "POPFD"
  Data$ "POR"
  Data$ "PSLLD"
  Data$ "PSLLQ"
  Data$ "PSLLW"
  Data$ "PSRAD"
  Data$ "PSRAW"
  Data$ "PSRLD"
  Data$ "PSRLQ"
  Data$ "PSRLW"
  Data$ "PSUBB"
  Data$ "PSUBD"
  Data$ "PSUBSB"
  Data$ "PSUBSW"
  Data$ "PSUBUSB"
  Data$ "PSUBUSW"
  Data$ "PSUBW"
  Data$ "PUNPCKHBW"
  Data$ "PUNPCKHDQ"
  Data$ "PUNPCKHWD"
  Data$ "PUNPCKLBW"
  Data$ "PUNPCKLDQ"
  Data$ "PUNPCKLWD"
  Data$ "PUSH"
  Data$ "PUSHA"
  Data$ "PUSHAD"
  Data$ "PUSHF"
  Data$ "PUSHFD"
  Data$ "PXOR"
  Data$ "RCL"
  Data$ "RCR"
  Data$ "RDMSR"
  Data$ "RDPMC"
  Data$ "RDTSC"
  Data$ "REP"
  Data$ "REPE"
  Data$ "REPNE"
  Data$ "REPNZ"
  Data$ "REPZ"
  Data$ "RET"
  Data$ "RETF"
  Data$ "ROL"
  Data$ "ROR"
  Data$ "RSM"
  Data$ "SAHF"
  Data$ "SAL"
  Data$ "SAR"
  Data$ "SBB"
  Data$ "SCAS"
  Data$ "SCASB"
  Data$ "SCASD"
  Data$ "SCASW"
  Data$ "SETA"
  Data$ "SETAE"
  Data$ "SETB"
  Data$ "SETBE"
  Data$ "SETC"
  Data$ "SETE"
  Data$ "SETG"
  Data$ "SETGE"
  Data$ "SETL"
  Data$ "SETLE"
  Data$ "SETNA"
  Data$ "SETNAE"
  Data$ "SETNB"
  Data$ "SETNBE"
  Data$ "SETNC"
  Data$ "SETNE"
  Data$ "SETNG"
  Data$ "SETNGE"
  Data$ "SETNL"
  Data$ "SETNLE"
  Data$ "SETNO"
  Data$ "SETNP"
  Data$ "SETNS"
  Data$ "SETNZ"
  Data$ "SETO"
  Data$ "SETP"
  Data$ "SETPE"
  Data$ "SETPO"
  Data$ "SETS"
  Data$ "SETZ"
  Data$ "SGDT"
  Data$ "SHL"
  Data$ "SHLD"
  Data$ "SHR"
  Data$ "SHRD"
  Data$ "SIDT"
  Data$ "SLDT"
  Data$ "SMSW"
  Data$ "STC"
  Data$ "STD"
  Data$ "STI"
  Data$ "STOS"
  Data$ "STOSB"
  Data$ "STOSD"
  Data$ "STOSW"
  Data$ "STR"
  Data$ "SUB"
  Data$ "TEST"
  Data$ "UD2"
  Data$ "VERR"
  Data$ "VERW"
  Data$ "WAIT"
  Data$ "WBINVD"
  Data$ "WRMSR"
  Data$ "XADD"
  Data$ "XCHG"
  Data$ "XLAT"
  Data$ "XLATB"
  Data$ "XOR"

EndDataSection
; IDE Options = PureBasic 4.40 Beta 2 (MacOS X - x86)
; CursorPosition = 483
; FirstLine = 463
; Folding = --
; UseMainFile = CodeArchivViewer_Main.pb