Structure compilerinfo
  Keyword.s
  Len.l
EndStructure

Global Dim source.compilerinfo(15)
Restore SourceKeywords
For a=0 To 14
  Read.s source(a)\Keyword
  Read.l source(a)\Len
Next


DataSection
SourceKeywords:
  Data.s "; EnableNT4"
  Data.l 11
  Data.s "; EnableAsm"
  Data.l 11
  Data.s "; EnableXP"
  Data.l 10
  Data.s "; EnableOnError"
  Data.l 15
  Data.s "; DisableDebugger"
  Data.l 17
  Data.s "; Executable="
  Data.l 13
  Data.s "; ExecutableFormat=" 
  Data.l 19
  Data.s "; CommandLine="      
  Data.l 14
  Data.s "; UseIcon="  
  Data.l 10
  Data.s "; UseMainFile="      
  Data.l 14
  Data.s "; CPU="              
  Data.l 6
  Data.s "; CursorPosition="   
  Data.l 17
  Data.s "; FirstLine="        
  Data.l 12
  Data.s "; IDE Options ="
  Data.l 15
  Data.s "; Folding ="        
  Data.l 11
EndDataSection


Procedure CheckForSourceKeywords(a$)
  Protected b, c
  ; we don't show the source settings in the editor window
  b = #False
  c = 0
  Repeat
    If Left(a$, source(c)\Len) = source(c)\Keyword : b=#True : EndIf
    c + 1
  Until b=#True Or c > 14
  ProcedureReturn b
EndProcedure


; ==================================================

; Required procedures. They need not do anything

; dummy structure
Structure SourceFile
  EnableASM.l
EndStructure

#FILE_LoadFunctions = 0
#FILE_LoadAPI = 1
#DEFAULT_FunctionFile       = ""
#DEFAULT_ApiFile            = ""

; Procedure AddTools_Execute(Trigger, *Source.SourceFile)
; EndProcedure
; 
; Procedure ChangeStatus(Message$)
; EndProcedure
; 
; Procedure.s GenerateQuickHelpText(Word$)
; EndProcedure

XIncludeFile "CodeArchivViewer_HilightningEngine.pb"

;XIncludeFile "HilightningEngine.pb"

; ==================================================

EnableColoring = 1        ; set the hilightning options to on.
EnableCaseCorrection = 1
EnableKeywordBolding = 1

; init functions must be called first
InitSyntaxCheckArrays()
InitSyntaxHilightning()

; callback for the hilightning engine
Procedure HilightCallback(*StringStart.BYTE, Length, *Color, IsBold, TextChanged)
  Shared *OutCursor.BYTE

  If IsBold
    CopyMemoryString("\cf"+Str(*Color)+"\b ")
  Else
    CopyMemoryString("\cf"+Str(*Color)+" ")
  EndIf  
  
  ; add the string to the buffer and escape needed characters..
  ;
  While Length > 0
    
    If *StringStart\b = 13 Or *StringStart\b = 10  ; we have a newline here
      If PeekB(*StringStart+1) = 10 Or PeekB(*StringStart+1) = 13
        *StringStart + 1
        Length - 1
      EndIf
      CopyMemoryString("\line ")
      
    Else
    
      If *StringStart\b = '\' Or *StringStart\b = '{' Or *StringStart\b = '}'
        *OutCursor\b = '\'
        *OutCursor + 1 
      EndIf
      
      *OutCursor\b = *StringStart\b
      *OutCursor + 1
           
    EndIf
    
    *StringStart + 1
    Length - 1    
  Wend
  
  *OutCursor\b = 0
  
  ; remove the bold again
  ;
  If IsBold
    If EditorFontStyle & #PB_Font_Bold
      CopyMemoryString("\cf1\b ")
    Else
      CopyMemoryString("\cf1\b0 ")
    EndIf
  EndIf

EndProcedure


Structure AdditionalStreamInfo
  Action.l  ; 0=streamin 1=streamout
  Buffer.l
  Length.l
EndStructure


; callback for the RTF streaming
;
Procedure RichEdit_StreamCallback(*StreamInfo.AdditionalStreamInfo, *Buffer, BufferLength.l, *Result.LONG)
  Result = 0

  If *StreamInfo\Action = 0  ; stream in
  
    If BufferLength < *StreamInfo\Length
      CopyMemory(*StreamInfo\Buffer, *Buffer, BufferLength)
      
      *StreamInfo\Buffer + BufferLength
      *StreamInfo\Length - BufferLength      
      *Result\l = BufferLength
    
    Else
      CopyMemory(*StreamInfo\Buffer, *Buffer, *StreamInfo\Length)
            
      *Result\l = *StreamInfo\Length
      *StreamInfo\Length = 0

    EndIf
  
  Else  ; stream out
  
    If BufferLength < *StreamInfo\Length
      CopyMemory(*Buffer, *StreamInfo\Buffer, BufferLength)
      
      *StreamInfo\Buffer + BufferLength
      *StreamInfo\Length - BufferLength      
      *Result\l = BufferLength
    
    Else
      CopyMemory(*Buffer, *StreamInfo\Buffer, *StreamInfo\Length)
            
      *Result\l = *StreamInfo\Length
      *StreamInfo\Length = 0

    EndIf
  
  EndIf

  ProcedureReturn Result
EndProcedure

; little helper for rtf colors
;
Procedure BuildRTFColor(Color)
  Static ColorIndex
  
  If Color = -1    ; reset the curent color index
    ColorIndex = 0
  EndIf
  
  ColorIndex + 1

  CopyMemoryString("\red"+Str(Color & $FF))
  CopyMemoryString("\green"+Str((Color >> 8) & $FF))
  CopyMemoryString("\blue"+Str((Color >> 16) & $FF))
  CopyMemoryString(";")

  ProcedureReturn ColorIndex
EndProcedure


; call the hilightning engine
;
Procedure HilightAll(Gadget)
  CompilerIf #PB_Compiler_OS = #PB_OS_Windows
    Shared *OutCursor.BYTE
  
    HilightGadgetID = GadgetID(Gadget)
  
    ; prepare the gadget
    SendMessage_(HilightGadgetID, #EM_EXGETSEL, 0, @Range.CHARRANGE)
    SendMessage_(HilightGadgetID, #WM_SETREDRAW, 0, 0)
    SendMessage_(HilightGadgetID, #EM_SETOPTIONS, #ECOOP_XOR, #ECO_AUTOHSCROLL)  ; Remove the autoscroll feature, as if the line is too long, the SETSEL will move back the scrollbar at the beginning of the line      
    SendMessage_(HilightGadgetID, #EM_HIDESELECTION , 1, 0)  
    
    InBufferLength = SendMessage_(HilightGadgetID, #WM_GETTEXTLENGTH, 0, 0) + 10
    
    *InBuffer = AllocateMemory(InBufferLength)
    *OutBuffer = AllocateMemory(InBufferLength * 20 + 200)
    OutBufferLength = InBufferLength * 20 + 200
    
    If *InBuffer And *OutBuffer  
      ; get the gadget content..
      ;
      StreamInfo.AdditionalStreamInfo
      StreamInfo\Action = 1
      StreamInfo\Buffer = *InBuffer
      StreamInfo\Length = InBufferLength
      
      Stream.EDITSTREAM
      Stream\dwCookie   = @StreamInfo
      Stream\pfnCallback= @RichEdit_StreamCallback()        
      InBufferLength = SendMessage_(HilightGadgetID, #EM_STREAMOUT, #SF_TEXT, @Stream)      
  
        
      ; build the header
      ; 
      *OutCursor = *OutBuffer
      CopyMemoryString("", @*OutCursor) ; initialize the buffer copying
      
      acp.l = GetACP_()
      If acp=0 : acp=1252 : EndIf ; Default if none available?
      CopyMemoryString("{\rtf1\ansi\ansicpg"+Str(acp)+"\uc1 \deff0{\colortbl;")
  
      BuildRTFColor(-1) ; reset the color counter
    
  ;     *NormalTextColor    = BuildRTFColor($000000) ; normal text must be the first!
  ;     *BasicKeywordColor  = BuildRTFColor($666600);     *CommentColor       = BuildRTFColor($AAAA00)
  ;     *ConstantColor      = BuildRTFColor($724B92)
  ;     *StringColor        = *NormalTextColor
  ;     *PureKeywordColor   = BuildRTFColor($666600)
  ;     *ASMKeywordColor    = *NormalTextColor
  ;     *OperatorColor      = *NormalTextColor
  ;     *StructureColor     = *NormalTextColor
  ;     *NumberColor        = *NormalTextColor
  ;     *PointerColor       = *NormalTextColor
  ;     *SeparatorColor     = *NormalTextColor
  ;     *LabelColor         = *NormalTextColor
  
      *NormalTextColor    = BuildRTFColor(prefs(19)) ; normal text must be the first!
      *BasicKeywordColor  = BuildRTFColor(prefs(20))
      *CommentColor       = BuildRTFColor(prefs(24))
      *ConstantColor      = BuildRTFColor(prefs(23))
      *StringColor        = *NormalTextColor
      *PureKeywordColor   = BuildRTFColor(prefs(21))   ; including API commands
      *ASMKeywordColor    = BuildRTFColor(prefs(22))
      *OperatorColor      = *NormalTextColor
      *StructureColor     = *NormalTextColor
      *NumberColor        = *NormalTextColor
      *PointerColor       = *NormalTextColor
      *SeparatorColor     = *NormalTextColor
      *LabelColor         = *NormalTextColor
  
      
    
      CopyMemoryString("}\b0\cf1 ")
    
        
      ; now call the hilightning engine
      ;
      HilightningEngine(*InBuffer, InBufferLength, -1, @HilightCallback(), 0)
      
      CopyMemoryString("}}")  ; complete the buffer
        
      ; now stream the buffer back into the gadget
      ;
      StreamInfo.AdditionalStreamInfo
      StreamInfo\Action = 0
      StreamInfo\Buffer = *OutBuffer
      StreamInfo\Length = *OutCursor - *OutBuffer
  
      Stream.EDITSTREAM
      Stream\dwCookie   = @StreamInfo
      Stream\pfnCallback= @RichEdit_StreamCallback()        
      SendMessage_(HilightGadgetID, #EM_STREAMIN, #SF_RTF, @Stream)
    
      FreeMemory(*InBuffer)
      FreeMemory(*OutBuffer)
    EndIf
      
    ; reset selection and redraw state
    SendMessage_(HilightGadgetID, #EM_EXSETSEL, 0, @Range) ; Put the pointer to it's right place...
    SendMessage_(HilightGadgetID, #EM_HIDESELECTION, 0, 0)
    SendMessage_(HilightGadgetID, #EM_SETOPTIONS, #ECOOP_OR, #ECO_AUTOHSCROLL)
    SetFocus_   (HilightGadgetID)   
       
    SendMessage_(HilightGadgetID, #WM_SETREDRAW, 1, 0)       
    InvalidateRect_(HilightGadgetID, 0, 1)   
  CompilerElse
    Debug "HilightAll() procedure:"
    Debug "The Highlightning engine is only available on MS Windows for now..."
  CompilerEndIf
EndProcedure

; ================================================



; ; 
; Dim ValidCharacters.b(255)  ; used by Syntax Highlighting 
;   ; Build the ValidCharacter table to have have a faster routine to detect
;   ; if the char is a considered as a string or not. Nice improvement :)
;   ;
;   For k='0' To '9'            ; Tell that the ASCII from 0 to 9 are valid
;     ValidCharacters(k) = 1
;   Next
; 
;   For k='A' To 'Z'            ; Tell that the ASCII from A to Z are valid
;     ValidCharacters(k) = 1
;   Next
; 
;   For k='a' To 'z'            ; Tell that the ASCII from a to z are valid
;     ValidCharacters(k) = 1
;   Next
; 
;   ValidCharacters('_') = 1
; 
; 
;   ValidCharacters('#') = 1    ; added by Andre, for recognizing constants
; 


; ******************************************************
; Following code was taken from the CodeArchive example:
; EditorGadget_Color+Styles.pb by Freak
; ******************************************************
; Selects Text inside an EditorGadget 
; Line numbers range from 0 to CountGadgetItems(#Gadget)-1 
; Char numbers range from 1 to the length of a line 
; Set Line numbers to -1 to indicate the last line, and Char 
; numbers to -1 to indicate the end of a line 
; selecting from 0,1 to -1, -1 selects all. 

 
CompilerIf #PB_Compiler_OS = #PB_OS_Windows
  Structure CHARFORMAT2_ 
    cbSize.l 
    dwMask.l  
    dwEffects.l  
    yHeight.l  
    yOffset.l  
    crTextColor.l  
    bCharSet.b  
    bPitchAndFamily.b  
    szFaceName.b[#LF_FACESIZE]  
    _wPad2.w  
    wWeight.w  
    sSpacing.w  
    crBackColor.l  
    lcid.l  
    dwReserved.l  
    sStyle.w  
    wKerning.w  
    bUnderlineType.b  
    bAnimation.b  
    bRevAuthor.b  
    bReserved1.b 
  EndStructure 
CompilerEndIf
 
; Procedure Editor_BackColor(Gadget, Color.l) 
;   format.CHARFORMAT2_ 
;   format\cbSize = SizeOf(CHARFORMAT2_) 
;   format\dwMask = $4000000  ; = #CFM_BACKCOLOR 
;   format\crBackColor = Color 
;   SendMessage_(GadgetID(Gadget), #EM_SETCHARFORMAT, #SCF_SELECTION, @format) 
; EndProcedure

Procedure Editor_Select(Gadget, LineStart.l, CharStart.l, LineEnd.l, CharEnd.l)    
  CompilerIf #PB_Compiler_OS = #PB_OS_Windows
    sel.CHARRANGE 
    sel\cpMin = SendMessage_(GadgetID(Gadget), #EM_LINEINDEX, LineStart, 0) + CharStart - 1 
    
    If LineEnd = -1 
      LineEnd = SendMessage_(GadgetID(Gadget), #EM_GETLINECOUNT, 0, 0)-1 
    EndIf 
    sel\cpMax = SendMessage_(GadgetID(Gadget), #EM_LINEINDEX, LineEnd, 0) 
    
    If CharEnd = -1 
      sel\cpMax + SendMessage_(GadgetID(Gadget), #EM_LINELENGTH, sel\cpMax, 0) 
    Else 
      sel\cpMax + CharEnd - 1 
    EndIf 
    SendMessage_(GadgetID(Gadget), #EM_EXSETSEL, 0, @sel) 
  CompilerElse
    Debug "Editor_Select() procedure:"
    Debug "Marking of selected words is only available on MS Windows for now..."
  CompilerEndIf
EndProcedure 
; *******************************************************


; Procedure EditorGadgetLocate(Gadget,x,y) 
;   ; Set cursor position 
;   REG = GadgetID(Gadget) 
;   CharIdx = SendMessage_(REG,#EM_LINEINDEX,y-1,0) 
;   LLength = SendMessage_(REG,#EM_LINELENGTH,CharIdx,0) 
;   If LLength >= x-1 
;     CharIdx + x-1 
;   EndIf 
;   Range.CHARRANGE 
;   Range\cpMin = CharIdx 
;   Range\cpMax = CharIdx 
;   SendMessage_(REG,#EM_EXSETSEL,0,Range) 
; EndProcedure 
; 


; ***************** source from syntaxhighlighting userlib of DarkDragon *******************
Procedure EditorGadgetLocate(Gadget,x,y,length) 
  ; Set cursor position 
  CompilerIf #PB_Compiler_OS = #PB_OS_Windows
    REG = GadgetID(Gadget) 
    CharIdx = SendMessage_(REG,#EM_LINEINDEX,y-1,0) 
    LLength = SendMessage_(REG,#EM_LINELENGTH,CharIdx,0) 
    If LLength >= x-1 
      CharIdx + x-1 
    EndIf 
    Range.CHARRANGE 
    Range\cpMin = CharIdx 
    Range\cpMax = CharIdx+length
    SendMessage_(REG,#EM_EXSETSEL,0,Range) 
  CompilerElse
    Debug "EditorGadgetLocate() procedure:"
    Debug "Locating a specific position in EditorGadget is only available on MS Windows for now..."
  CompilerEndIf
EndProcedure 

Procedure EditorGadgetBackColor(Gadget, Color.l) 
  CompilerIf #PB_Compiler_OS = #PB_OS_Windows
    format.CHARFORMAT2_ 
    format\cbSize = SizeOf(CHARFORMAT2_) 
    format\dwMask = $4000000  ; = #CFM_BACKCOLOR 
    format\crBackColor = Color 
    SendMessage_(GadgetID(Gadget), #EM_SETCHARFORMAT, #SCF_SELECTION, @format) 
  CompilerElse
    Debug "EditorGadgetBackColor() procedure:"
    Debug "Setting background color in EditorGadget is only available on MS Windows for now..."
  CompilerEndIf
EndProcedure


; *******************************************************************************************

Procedure HighlightSource(line$,line)

;   CPos.CHARRANGE
;   SendMessage_(GadgetID(#EditorSource), #EM_EXGETSEL,0,CPos)
;   SendMessage_(GadgetID(#EditorSource), #EM_HIDESELECTION, 1, 0)



  ForEach searchterms()
    If searchterms()\valid < 2     ; must be 0 or 1  (OR, AND) search terms, because 2 (NOT) terms wouldn't have resulted in found source files
      pos = 1
      len = Len(searchterms()\term)
      Repeat
        pos = FindString(LCase(line$),searchterms()\term,pos)
        If pos
          lastpos = pos + len
;          If Len(line$) < lastpos      ; this check is needed, because if the colored search term is at the end of a sourceline, the coloring affects also the next line
;            Debug "Sonderfall"
;            lastpos - 1
;          EndIf
          ; ********* original code ***********
;           Editor_Select(#EditorSource,line,pos,line,lastpos)
;           Editor_BackColor(#EditorSource, RGB(255,200,100)) 
          ; ***********************************


      EditorGadgetLocate(#EditorSource,pos,line+1,len)
      EditorGadgetBackColor(#EditorSource, prefs(26))   ;RGB(255,200,100)



          pos + 1
        EndIf
      Until pos = 0
    EndIf  
  Next

EndProcedure




Procedure LoadSource(file$)
;Debug file$
  SetGadgetText(#EditorSource,"")
  
  If ReadFile(0, file$)
    While Eof(0) = 0
      a$ = ReadString(0) 
      lines + 1
      If CheckForSourceKeywords(a$) = #False        ; we don't show the source settings in the editor window
        AddGadgetItem(#EditorSource,-1,a$+" ")
        
;        HighlightSource(a$,a-1)     ; use current string of read source line and its line number as parameter
      EndIf
    Wend
    CloseFile(0)

    If prefs(18) = 1     ; syntax-highlighting enabled ?
      HilightAll(#EditorSource)
    EndIf


    ; Highlight the background of a found search item....
    For a = 1 To lines-1
      a$ = GetGadgetItemText(#EditorSource,a-1,0)
      HighlightSource(a$,a-1)
    Next      

;     ; Some tests....
;     HighlightSource("a",4)     ; use current string of read source line and its line number as parameter
;     HighlightSource("a",5)     ; use current string of read source line and its line number as parameter
;     HighlightSource("a",6)     ; use current string of read source line and its line number as parameter
;     HighlightSource("a",7)     ; use current string of read source line and its line number as parameter
; 



;  On this way the SyntaxHighlighting is done after adding all source lines to the editor gadget:
;    for this you can use SyntaxHighlighting2() procedure - a full source procedure
;    or alternatively the SyntaxHighlighting3() procedure - a 'wrapper' using the standard SyntaxHighLighting() procedure
;    If prefs(18) = 1
;      SyntaxHighlighting2()
;    EndIf
; 

;    gad.l = GadgetID(#EditorSource)
;    SendMessage_(gad,#EM_REPLACESEL,0,Text$)   ; Insert a line of text in the editorgadget

    EditorGadgetLocate(#EditorSource,1,1,0)            ; set the cursor to the top-left position


  EndIf
EndProcedure

; IDE Options = PureBasic 4.40 Beta 2 (MacOS X - x86)
; CursorPosition = 49
; FirstLine = 48
; Folding = --
; UseMainFile = CodeArchivViewer_Main.pb